/*
 * Example: the Lao language.
 * Copyright (C) 2000 Leandro Bernsmller
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#include <stdio.h>
#include <strings.h>
#include "../../../include/stdcomp.h"



int main ( int argc, char** argsv [])
{
  // define from where and to where data flow
  FILE* data_source = stdin;
  FILE* error_dest = stderr;
  FILE* data_dest = stdout;

  // define character classes
  class_chclass_table ct;
  ct.init ();
  ct.set_max_enumerations (23);
  ct.add_enumeration (0, 31, CHC_CONTROLS);
  ct.add_enumeration (0, 32, CHC_SEPARATORS);
  ct.add_enumeration ('#', '#', CHC_INVALID_STRING_START);
  ct.add_enumeration ('(', '(', CHC_INVALID_STRING_START);
  ct.add_enumeration (')', ')', CHC_INVALID_STRING_START);
  ct.add_enumeration ('{', '{', CHC_INVALID_STRING_START);
  ct.add_enumeration ('}', '}', CHC_INVALID_STRING_START);
  ct.add_enumeration ('[', '[', CHC_INVALID_STRING_START);
  ct.add_enumeration (']', ']', CHC_INVALID_STRING_START);
  ct.add_enumeration ('<', '<', CHC_INVALID_STRING_START);
  ct.add_enumeration ('>', '>', CHC_INVALID_STRING_START);
  ct.add_enumeration (34, 34, CHC_INVALID_STRING_START);
  ct.add_enumeration (39, 39, CHC_INVALID_STRING_START);
  ct.add_enumeration (39, 39, CHC_CHAR_DELIMITER);
  ct.add_enumeration (34, 34, CHC_STRING_DELIMITER);
  ct.add_enumeration ('(', '(', CHC_OPEN_PARENTHESIS);
  ct.add_enumeration (')', ')', CHC_CLOSE_PARENTHESIS);
  ct.add_enumeration ('{', '{', CHC_OPEN_BRACE);
  ct.add_enumeration ('}', '}', CHC_CLOSE_BRACE);
  ct.add_enumeration ('[', '[', CHC_OPEN_BRACKET);
  ct.add_enumeration (']', ']', CHC_CLOSE_BRACKET);
  ct.add_enumeration ('<', '<', CHC_OPEN_RELATIONAL);
  ct.add_enumeration ('>', '>', CHC_CLOSE_RELATIONAL);
  
  // setup history
  class_sbase_history hist;
  hist.init ();
  
  // define standard messages
  BOOL res;
  res = hist.predefine_messages ();
  if (!res) {
    if (hist.has_fatal_error ()) printf ("Fatal error.\n");
    printf ("Lao initialization error: can't setup history.");
    return 1;
  }
  
  // setup enviroment
  class_enviroment env;
  env.init ();
  
  // define macro to test errors while building a component
#define TEST_BUILD_ERROR \
  if ( hist.has_exit() || hist.has_fatal_error () ) { \
    if (hist.has_fatal_error ()) printf ("Fatal error.\n"); \
      hist.show ( data_dest ); \
      return hist.get_exit_code (); \
  }

  // init standard components
  class_component *comp;
  comp = ::build_component ( 0, &env, &hist, ::call_stdcomp_exit, "<user>", "exit" );
  TEST_BUILD_ERROR;
  env.insert ( "exit", comp );
  
  // setup script interpreter
  
  // define organizers
  class_translator translator;
  translator.init ();
  class_component *parcomp;
  parcomp = ::build_component ( 0, &env, &hist, ::call_plist, "<user>", "plist" );
  TEST_BUILD_ERROR;
  translator.add_type ( "list", parcomp );
  parcomp = ::build_component ( 0, &env, &hist, ::call_pchar, "<user>", "pchar" );
  TEST_BUILD_ERROR;
  translator.add_type ( "char", parcomp );
  parcomp = ::build_component (0, &env, &hist, ::call_pstring, "<user>", "pstring");
  TEST_BUILD_ERROR;
  translator.add_type ( "string", parcomp );
  
  // define executor
  class_executor executor;
  executor.init ();

  // for each input line
  while (! feof ( data_source ) && ! hist.has_exit () ) {
    
    // display the prompt signal                
    printf ("? ");
                
    // parse input              
    class_component* tr_comp;
    tr_comp = translator.translate_input ( data_source, &ct, &env, &hist );

    // verify if happens a serious error
    if (hist.has_fatal_error ()) {
      printf ("Fatal error.\n");
      return hist.get_exit_code ();
    }

    // try interpret the translated input
    if (! hist.has_exit () && tr_comp) {
      
      // mostra representacao textual do script
      //printf ("\nlistagem do script:\n");
      //script.show ( &env );

      // interpret script
      class_component* res_comp;
      executor.interpret ( &env, &hist, tr_comp, &res_comp );

      // verify if happens a serious error
      if (hist.has_fatal_error ()) {
        printf ("Fatal error.\n");
        return hist.get_exit_code ();
      }

      // show textual representation of the given component
      if (res_comp) {

        class_string *str = (class_string *) res_comp->call (BIN_GET_TEXT_REPRESENTATION, &env, &hist, 0);
        
        if (str)
          printf ("%s\n", str->get ());
        else
          // if an serious error happens, abort
          if (hist.has_fatal_error ()) {
            printf ("Fatal error.\n");
            return hist.get_exit_code ();
          }
        
      } else {
        class_string *str = (class_string *) tr_comp->call (BIN_GET_TEXT_REPRESENTATION, &env, &hist, 0);
        printf ("Warning: translated component '%s' has no activation result.\n",str->get());
      }
    }
    
    // show history messages
    hist.show ( data_dest );
  }

  int script_exit_code = hist.get_exit_code ();

  // clear the script
  translator.terminate (&env, &hist);
  executor.terminate (&env, &hist);
  env.terminate ();
  hist.terminate ();
  ct.terminate ();
        
  return script_exit_code;
}
