/*
 * Example: basic use of lists.
 * Copyright (C) 2000  Leandro Bernsmller
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#include <stdio.h>
#include <string.h>
#include "../../../include/base.h"



// used in some samples:
struct my_record {
	int   id;
	char *str;
};

DATAPT record_comparator (DATAPT data1, DATAPT data2);



int main ()
{
	BOOL  success;		// flag errors when operationg lists

	// declare one list

	class_list test;

	// show list status

	printf ("total nodes: %d\n", test.get_total_nodes ());

	// add node

	puts ("adding node to list...");

	// insert node in list
	char *my_text = "this is a test!!!";
	// observation: when passing a variable or pointer to list use 'get_from'
	// before name.
	success = test.insert_first (BEFORE, get_from my_text);
	if (!success)
		exit (1);	// test for error

	// show list status

	printf ("total nodes: %d\n", test.get_total_nodes ());

	// show node

	char *got_text = 0;
	// observation: when a variable or pointer would change acording
	// list data, use 'put_in' before name.
	success = test.get_first (put_in got_text);
	if (!success)
		exit (1);
	printf ("node data: '%s'\n", got_text);

	// remove node from list

	test.remove_all ();
	puts ("node removed...");

	// add 3 nodes

	puts ("adding 3 nodes to list...");

	success = test.insert_first (BEFORE, 3);	// insert 3

	if (!success)
		exit (1);	// test success

	success = test.insert_first (BEFORE, 2);	// insert 2

	if (!success)
		exit (1);	// test success

	success = test.insert_first (BEFORE, 1);	// insert 1

	if (!success)
		exit (1);	// test success

	// show list status

	printf ("total nodes: %d\n", test.get_total_nodes ());

	// show all nodes

	printf ("nodes data:");

	test.select_first ();
	while (test.has_selection ()) {

		char  value;
		success = test.get_selected (put_in value);
		if (!success) {
			puts ("error when reading list...");
			break;
		}
		printf (" %d", value);
		test.select_next ();
	}
	puts (".");

	// revert list

	puts ("reverting list...");

	test.revert ();

	// show all nodes

	printf ("nodes data:");

	test.select_first ();
	while (test.has_selection ()) {

		int   value;
		success = test.get_selected (put_in value);
		if (!success) {
			puts ("error when reading list...");
			break;
		}
		printf (" %d", value);
		test.select_next ();
	}
	puts (".");

	// filling other list

	class_list test2;

	puts ("filling second list with 4 nodes...");

	success = test2.insert_last (AFTER, 8);		// hey, another insertion way!

	if (!success)
		exit (1);
	success = test2.insert_last (AFTER, 5);
	if (!success)
		exit (1);
	success = test2.insert_last (AFTER, 7);
	if (!success)
		exit (1);
	success = test2.insert_last (AFTER, 4);
	if (!success)
		exit (1);

	// show all second list nodes

	printf ("second list nodes data:");

	test2.select_first ();
	while (test2.has_selection ()) {

		int   value;
		success = test2.get_selected (put_in value);
		if (!success) {
			puts ("error when reading list...");
			break;
		}
		printf (" %d", value);
		test2.select_next ();
	}
	puts (".");

	// appending lists

	puts ("appending second list to first list...");

	test.append_last (AFTER, test2);

	puts ("second list can't be used anymore, only first list is valid.");

	// show all nodes

	printf ("nodes data:");

	test.select_first ();
	while (test.has_selection ()) {

		int   value;
		success = test.get_selected (put_in value);
		if (!success) {
			puts ("error when reading list...");
			break;
		}
		printf (" %d", value);
		test.select_next ();
	}
	puts (".");

	// sorting list

	puts ("sorting...");

	test.sort ();

	// show all nodes

	printf ("nodes data:");

	test.select_first ();
	while (test.has_selection ()) {

		int   value;
		success = test.get_selected (put_in value);
		if (!success) {
			puts ("error when reading list...");
			break;
		}
		printf (" %d", value);
		test.select_next ();
	}
	puts (".");

	// finding one node

	puts ("looking for node with data 3...");

	DWORD num = test.find_sorted (AFTER, 3);
	if (num)
		puts ("node found and selected.");
	else
		puts ("node not found.");

	// showing selected node position

	DWORD index;
	if (test.get_select_position (index))
		printf ("selected node at position: %d.\n", index);
	else
		puts ("none selected.");

	// removing selected node

	puts ("removing selected node...");

	int   data;
	test.remove_selected (put_in data);
	printf ("removed node data: %d.\n", data);

	// show all nodes

	printf ("nodes data:");

	test.select_first ();
	while (test.has_selection ()) {

		int   value;
		success = test.get_selected (put_in value);
		if (!success) {
			puts ("error when reading list...");
			break;
		}
		printf (" %d", value);
		test.select_next ();
	}
	puts (".");

	// finding node again

	int   look = 3;
	printf ("looking for node with data %d...\n", look);

	num = test.find_sorted (BEFORE, look);
	if (num)
		puts ("node found and selected.");
	else
		puts ("node not found, selected closest node.");

	// showing selected node position and data

	if (test.get_select_position (index)) {

		printf ("selected node at position: %d", index);

		int   data;
		if (test.get_selected (put_in data))
			printf (" with data: %d", data);

		puts (".");
	}
	else
		puts ("none selected.");

	// insert one node keeping list sorted

	puts ("inserting data keeping list sorted...");

	int   new_data = 6;
	printf ("inserting: %d.\n", new_data);
	// discover the best position to insert, selecting the closest.
	test.find_sorted (AFTER, get_from new_data);
	// insert
	test.insert_selected (BEFORE, get_from new_data);

	// show all nodes

	printf ("nodes data:");

	test.select_first ();
	while (test.has_selection ()) {

		int   value;
		success = test.get_selected (put_in value);
		if (!success) {
			puts ("error when reading list...");
			break;
		}
		printf (" %d", value);
		test.select_next ();
	}
	puts (".");

	// updatting: summing data with 1, two times

	puts ("updatting: summing list data with 1 (two times)...");

	test.do_circular ();
	test.select_first ();
	int   counter = 0;
	int   end_count = 2 * test.get_total_nodes ();
	while (test.has_selection () && counter != end_count) {

		// get
		int   value;
		success = test.get_selected (put_in value);
		if (!success) {
			puts ("error when reading list...");
			break;
		}

		// sum
		value += 1;

		// update
		test.swap_selected (put_in value);

		// find next
		test.select_next ();
		counter++;
	}
	test.do_linear ();

	// show all nodes

	printf ("nodes data:");

	test.select_first ();
	while (test.has_selection ()) {

		int   value;
		success = test.get_selected (put_in value);
		if (!success) {
			puts ("error when reading list...");
			break;
		}
		printf (" %d", value);
		test.select_next ();
	}
	puts (".");

	// splitting list

	puts ("splitting list...");

	class_list other;
	int   end = test.get_total_nodes () - 1;	// last node index

	test.split (3, end, other);

	// show all nodes

	printf ("first list data:");

	test.select_first ();
	while (test.has_selection ()) {

		int   value;
		success = test.get_selected (put_in value);
		if (!success) {
			puts ("error when reading list...");
			break;
		}
		printf (" %d", value);
		test.select_next ();
	}
	puts (".");

	printf ("other list data:");

	other.select_first ();
	while (other.has_selection ()) {

		int   value;
		success = other.get_selected (put_in value);
		if (!success) {
			puts ("error when reading list...");
			break;
		}
		printf (" %d", value);
		other.select_next ();
	}
	puts (".");

	// clear the lists

	puts ("cleaning lists...");

	test.remove_all ();
	other.remove_all ();

	// examples using more complex structures

	puts ("");
	puts ("examples using more complex structures:");

	class_list base;

	// add data created dynamically

	puts ("adding data...");

	// create record
	my_record *rec = (my_record *) malloc (sizeof (my_record));
	if (!rec)
		exit (1);
	rec->id = 9;
	rec->str = "foo";

	// add to list
	success = base.insert_at_position (AFTER, 0, get_from rec);
	if (!success)
		exit (1);

	// add more records
	// add one
	rec = (my_record *) malloc (sizeof (my_record));
	if (!rec)
		exit (1);
	rec->id = 6;
	rec->str = "bar";
	success = base.insert_at_position (AFTER, 1, get_from rec);
	if (!success)
		exit (1);

	// add two
	rec = (my_record *) malloc (sizeof (my_record));
	if (!rec)
		exit (1);
	rec->id = 1;
	rec->str = "foxes";
	success = base.insert_at_position (AFTER, 1, get_from rec);
	if (!success)
		exit (1);

	// add three
	rec = (my_record *) malloc (sizeof (my_record));
	if (!rec)
		exit (1);
	rec->id = 5;
	rec->str = "foo";
	success = base.insert_at_position (AFTER, 1, get_from rec);
	if (!success)
		exit (1);

	// add four
	rec = (my_record *) malloc (sizeof (my_record));
	if (!rec)
		exit (1);
	rec->id = 0;
	rec->str = "jump";
	success = base.insert_at_position (AFTER, 1, get_from rec);
	if (!success)
		exit (1);

	// add five
	rec = (my_record *) malloc (sizeof (my_record));
	if (!rec)
		exit (1);
	rec->id = 3;
	rec->str = "zuzza";
	success = base.insert_at_position (AFTER, 1, get_from rec);
	if (!success)
		exit (1);

	// add six
	rec = (my_record *) malloc (sizeof (my_record));
	if (!rec)
		exit (1);
	rec->id = 2;
	rec->str = "foo";
	success = base.insert_at_position (AFTER, 1, get_from rec);
	if (!success)
		exit (1);

	// show list

	puts ("list data:");

	base.select_first ();
	while (base.has_selection ()) {

		my_record *data_rec;
		success = base.get_selected (put_in data_rec);
		if (!success) {
			puts ("error when reading list...");
			break;
		}
		printf (" %d %s\n", data_rec->id, data_rec->str);
		base.select_next ();
	}

	// set compare function

	puts ("setting compare function...");

	base.set_compare_function (record_comparator);

	// finding data

	puts ("data search in unsorted list.");

	// define record to look for
	my_record *ref_rec = (my_record *) malloc (sizeof (my_record));
	memset (ref_rec, 0, sizeof (my_record));
	ref_rec->str = (char *) malloc (10);
	memset (ref_rec->str, 0, 10);
	strcpy (ref_rec->str, "foo");

	printf ("finding record with '%s'...\n", ref_rec->str);

	base.select_none ();
	base.find_unsorted (AFTER, get_from ref_rec);

	// showing selected node

	if (base.has_selection ()) {

		printf ("node found and selected");

		my_record *rec;
		base.get_selected (put_in rec);
		printf (", data: %d %s.\n", rec->id, rec->str);
	}
	else
		puts ("none selected, node not found.");

	// finding all records

	puts ("finding all match...");

	base.select_none ();
	do {

		// find
		base.find_unsorted (AFTER, get_from ref_rec);

		// show
		my_record *rec;
		success = base.get_selected (put_in rec);
		if (success)
			printf (" found: %d %s.\n", rec->id, rec->str);
	} while (success);

	// sort

	puts ("sortting alphabetically...");

	base.sort ();

	// show list

	puts ("list data:");

	base.select_first ();
	while (base.has_selection ()) {

		my_record *data_rec;
		success = base.get_selected (put_in data_rec);
		if (!success) {
			puts ("error when reading list...");
			break;
		}
		printf (" %d %s\n", data_rec->id, data_rec->str);
		base.select_next ();
	}

	// find all match

	puts ("data search in sorted list.");

	printf ("remove all matching '%s'.\n", ref_rec->str);

	DWORD num_found = base.find_sorted (BEFORE, get_from ref_rec);

	printf ("found %d matching nodes.\n", num_found);

	for (DWORD count = 0; count < num_found; count++) {
		DATAPT tmp;
		base.remove_selected (put_in tmp);
		base.select_next ();
	}

	// show list

	puts ("list data:");

	base.select_first ();
	while (base.has_selection ()) {

		my_record *data_rec;
		success = base.get_selected (put_in data_rec);
		if (!success) {
			puts ("error when reading list...");
			break;
		}
		printf (" %d %s\n", data_rec->id, data_rec->str);
		base.select_next ();
	}

	// clean list and other data

	puts ("cleaning memory...");

	base.remove_all (TRUE);
	free (ref_rec->str);
	free (ref_rec);
}



/*
 * Compare function for records sample.
 */
DATAPT
record_comparator (DATAPT data1, DATAPT data2)
{
	my_record *rec1 = (my_record *) data1;
	my_record *rec2 = (my_record *) data2;

	return (DATAPT) strcmp (rec1->str, rec2->str);
}
