/*
 * Declare comp.cc
 * See source file for details.
 */



#ifndef __SBASE_COMPONENT_H__
#define __SBASE_COMPONENT_H__



// Pointer to activation function.
// Parameters: the class component object, system message,
// enviroment (table with names and components), script history,
// component data and an extra call parameter.
#define CALLFN(name) DATAPT (name) (class_component*, \
                                    DATAPT, \
                                    class_enviroment*, \
                                    class_sbase_history*, \
                                    DATAPT*, \
                                    DATAPT)
                                    

// Verify if component data was defined. Designed to be used during
// component activation. If data wasn't defined, then register the
// problem in history and stop component activation.
#define DATA_AVAILABLE(data, hist) \
    if ((data) == 00) { \
       (hist)->add_predef_text (LH_TEXT, "error10"); \
       (hist)->add_flag (LH_ERROR, ERR_NO_DATA); \
       (hist)->set_exception (LE_EXIT | LE_ERROR | LE_FATAL_ERROR); \
       return 00; \
    }
    
// Verify if component data was not defined. Designed to be used during
// component activation, in the oposit sence of the test about data
// availability. If data was defined, then register the
// problem in history and stop component activation.
#define DATA_UNAVAILABLE(data, hist) \
    if ((data) != 00) { \
       (hist)->add_predef_text (LH_TEXT, "error9"); \
       (hist)->add_flag (LH_ERROR, ERR_DATA_EXISTS); \
       (hist)->set_exception (LE_EXIT | LE_ERROR | LE_FATAL_ERROR); \
       return 00; \
    }

// Verify if component data points some place. Designed to be used during
// component activation. If data wasn't pointed (points NULL), then
// register the problem in history and stop component activation.
#define DATA_POINTED(pdata, hist) \
    if ((pdata) == 00) { \
       (hist)->add_predef_text (LH_TEXT, "error8"); \
       (hist)->add_flag (LH_ERROR, ERR_NO_DATA_REFERED); \
       (hist)->set_exception (LE_EXIT | LE_ERROR | LE_FATAL_ERROR); \
       return 00; \
    }
    
// Verify if the given variable points NULL, if yes supose memory
// is low. Desined to be used after a memory allocation. If points NULL
// then register the problem in history and stop component activation.
#define MEMORY_ALLOCATED(var, hist) \
    if ((var) == 00) { \
       (hist)->add_predef_text (LH_TEXT, "error2"); \
       (hist)->add_flag (LH_ERROR, ERR_NO_MEMORY); \
       (hist)->set_exception (LE_EXIT | LE_ERROR | LE_FATAL_ERROR); \
       return 00; \
    }



// C functions:
class_component *build_component (DATAPT,
                                  class_enviroment*,
				  class_sbase_history*,
				  CALLFN (*), char *, char *);



// base component
class class_component {

  public:

  class_component ();
  ~class_component ();

  protected:

  DATAPT m_data;
  char *m_parent_name;
  char *m_self_name;
  CALLFN (*m_call_function);

  // external user functions
  public:

  void  init ();
  void  terminate ();
  BOOL  create (char *, char *, CALLFN (*), class_sbase_history*);
  DATAPT call (DATAPT, class_enviroment*, class_sbase_history*, DATAPT);
  l_op_id parse_param_str (char*, DWORD, l_op_element*);
	char* get_parent_name ();
	char* get_self_name ();
};



// end declarations
#endif
