/*
 * Basic declarations.
 */



#ifndef __SBASE_DEF_H__
#define __SBASE_DEF_H__



class class_component;



// error codes
#define l_error DWORD
#define ERR_NONE 0
#define ERR_UNDEFINED 1
#define ERR_NO_MEMORY 2
#define ERR_NO_NAME 3
#define ERR_PARSE_CONTROL 4
#define ERR_PARSE_EOI 5
#define ERR_PARSE_EOL 6
#define ERR_PARSE_CHAR 7
#define ERR_NO_DATA_REFERED 8
#define ERR_DATA_EXISTS 9
#define ERR_NO_DATA 10



// debug modes
#define l_debug DWORD
#define DB_NONE 0
#define DB_SINTAX_PROCESSING 1
#define DB_BREAKPOINT 2
#define DB_TRACE 4
#define DB_TEXT_REPRESENTATION 8



// possible messages for components
enum l_sysmsg {
  NONE,
  BIN_CREATE_DATA,
  BIN_CLEAN_DATA,
  INVOKE_AS_REFERENCE,
  INVOKE,
  ACTIVATE_AS_REFERENCE,
  ACTIVATE,
  ACTIVATE_AS_PARAMETER,
  BIN_ACTIVATE,
  BIN_SYNC_RECOGNITION,
  BIN_PROCESS_SYMBOL,
  BIN_RECOGNITION_TOKEN_BEGIN,
  BIN_RECOGNITION_TOKEN_SUCC,
  BIN_RECOGNITION_FAIL,
  BIN_RETURN_COMPONENT,
  BIN_ADD_COMPONENT,
  BIN_GET_TEXT_REPRESENTATION,
  COPY
};



// Define one message flag with the system message to be passed
// to component and the required service.
// Service is a component dependent flag.
// sysmsg - 8 bits system message, defined in l_sysmsg
// service - 24 bits service identification (some are defined in l_binact_services)
#define set_msgsrv(sysmsg, service) (l_sysmsg (sysmsg | (service << 8)))

// get the main system message flag
#define get_sysmsg(msg) (msg & 0xff)

// get the required service from the system message
#define get_actsrv(msg) (msg >> 8)



// Name of parameters list component, in enviroment.
// Used to access parameters during activation.
#define PARAMS_NAME "params"



// possible activation operators for components
#define l_op_id unsigned long int

// parameters operators table, for componenters
struct l_op_element {
  l_op_id id;
  char*   str;
};



// Some common services components can provide (optionaly)
// in BIN_ACTIVATE system messages.
enum l_binact_services {
  GET_ELEMENT,
  NUMBER_OF_ELEMENTS,
  GET_INTEGER
};



// return a reference to one component from a list of parameters
// (the parameters are in a component list).
// The first parameter is the self (activated) component.
// index - component index
// param - DATAPT reference to a list component with the parameters
// env - the enviroment
// hist - the history
#define	paramv(index, param, env, hist) (class_component*) ( (class_component*) param)->call (set_msgsrv(BIN_ACTIVATE, GET_ELEMENT), env, hist, index)



// return the number of parameters (elements) from a given list
// The first parameter is the self (activated) component.
// param - DATAPT reference to a list component with the parameters
// env - the enviroment
// hist - the history
#define	paramc(param, env, hist) (DWORD) ( (class_component*) param)->call (set_msgsrv(BIN_ACTIVATE, NUMBER_OF_ELEMENTS), env, hist, 0)



// end declarations
#endif
