/*
 * Implement a symbol table, works as a discrete function which
 * relacionates charactes (symbols) with some classes.
 * Possible examples:
 * 0, 1, 2, 3 ,4, 5, 6, 7, 8, 9 - DIGITS
 * <, >, [, ], {, }, (, ) - structure_separators
 * 
 * Each set of chars, which belongs to one class, is known as an enumeration.
 * Each enumeration has information about its first and last characters (acording
 * ASCII order). The middle characters are considered part of the enumeration.
 * Enumerations has the identification of the character class, too.
 * If user want to specify what characters are digits, it need enter, for example:
 * 0 (the first), 9 (the last), and, 12 (one id representing character class DIGIT).
 * If the enumeration has only one character, use it as first and last.
 * 
 * LICENSE:
 *
 * Copyright (C) 2000 Leandro Bernsmller
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#include <stdlib.h>
#include "../../include/base/type.h"
#include "../../include/base/chctab.h"



class_chclass_table::class_chclass_table ()
{
  init ();
}



class_chclass_table::~class_chclass_table ()
{
  terminate ();
}



void  class_chclass_table::
init ()
{
  m_max_enumerations = 0;
  m_num_enumerations = 0;
  m_tab = 0;
}



void  class_chclass_table::
terminate ()
{
  if (! m_tab) free (m_tab);
  init ();
}



/*
 * Define the maximal number of enumerations.
 * return:
 * TRUE - ok; FALSE - no memory
 */
BOOL  class_chclass_table::
set_max_enumerations (DWORD num)
{
  terminate ();
  m_max_enumerations = num;
  m_tab = (chars_enum*) malloc (sizeof (chars_enum) * num);
  if (! m_tab) return FALSE;
  return TRUE;
}



/*
 * Add one enumeration.
 * first - first character in enumeration
 * last - the last one
 * classid - class identification
 *
 * return:
 * TRUE - ok; FALSE - error (not enough space for enumerations)
 */
BOOL  class_chclass_table::
add_enumeration (char first, char last, DWORD classid)
{
  // verify if has free space for more enumerations
  if (m_num_enumerations < m_max_enumerations) {
    chars_enum* ce = m_tab + m_num_enumerations;
    ce->first = first;
    ce->last = last;
    ce->classid = classid;
    m_num_enumerations ++;
    return TRUE;
  }

  return FALSE;
}



/*
 * Verify if the given character belongs to the given class identification.
 * Observation: each character can be in more than one enumeration (e.g. class).
 * ch - character to test
 * classid - the character class
 *
 * return:
 * TRUE - character is in class; FALSE - is not
 */
BOOL  class_chclass_table::
match (char ch, DWORD classid)
{
  chars_enum* en = m_tab;
  for (DWORD cpos = 0; cpos < m_num_enumerations; cpos ++)
    if (classid == en->classid &&
		    ch >= en->first &&
				ch <= en->last)
			return TRUE;
    else
      en ++;

  return FALSE;
}
