/*
 * Character component.
 * 
 * LICENSE:
 *
 * Copyright (C) 2000 Leandro Bernsmller
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#include <stdio.h>
#include "../../include/sbase.h"
#include "../../include/lao/cochar.h"



/*
 * Function to handle char messages.
 * self - class_component instance for this object
 * msg - lao system message ID
 * env - reference to name table
 * hist - reference to history
 * pdata - points component data
 * param - component parameter
 *
 * return:
 * message process result
 */
DATAPT
call_char (class_component * self, DATAPT msg, class_enviroment * env, 
           class_sbase_history* hist, DATAPT * pdata, DATAPT param)
{
    DATA_POINTED (pdata, hist);

    switch (get_sysmsg (msg)) {

    case BIN_CREATE_DATA:
      {
	DATA_UNAVAILABLE (*pdata, hist);
            
	*pdata = (DATAPT) malloc (sizeof (class_char));
	MEMORY_ALLOCATED (*pdata, hist);

	// define char information
	class_char *character = (class_char *) * pdata;
	character->init ();

	// expect: param as char* (string)
	char* str = (char*) param;
	character->set (str [0]);
	return *pdata;
      }
      break;

    case BIN_CLEAN_DATA:
      {

	DATA_AVAILABLE (*pdata, hist);

	class_char *character = (class_char *) * pdata;
	character->terminate ();
	free ((void *) *pdata);
	*pdata = 0;
	return *pdata;
      }
      break;

    case INVOKE:
    case INVOKE_AS_REFERENCE:
      {
	class_component *discardable_comp = (class_component *) ::call_char (self, COPY, env, hist, pdata, 0);
	if (!discardable_comp)
	  return 00;

	return discardable_comp->call (ACTIVATE, env, hist, param);
      }
      break;

    case ACTIVATE:
    case ACTIVATE_AS_REFERENCE:
      {
	DATA_AVAILABLE (*pdata, hist);

	class_char *ch = (class_char *) * pdata;
	return (DATAPT) self;
      }
      break;

    case ACTIVATE_AS_PARAMETER:
      return (DATAPT) self;
      break;

    case BIN_ACTIVATE:
    case BIN_PROCESS_SYMBOL:
    case BIN_SYNC_RECOGNITION:
    case BIN_RECOGNITION_TOKEN_BEGIN:
    case BIN_RECOGNITION_TOKEN_SUCC:
    case BIN_RECOGNITION_FAIL:
    case BIN_RETURN_COMPONENT:
    case BIN_ADD_COMPONENT:
      {
	return 00;
      }
      break;

    case BIN_GET_TEXT_REPRESENTATION:
      {
	class_string *str = (class_string *) malloc (sizeof (class_string));
	str->init ();
	class_char *ch = (class_char *) * pdata;
	str->append_char ('\'');
	str->append_char (ch->get ());
	str->append_char ('\'');
	return (DATAPT) str;
      }
      break;

    case COPY:
      {
	DATA_AVAILABLE (*pdata, hist);

	class_char *ch = (class_char *) * pdata;
	char str [2];
	str [0] = ch->get ();
	str [1] = 0;
	return (DATAPT) ::build_component ((DATAPT) str, env, hist, ::call_char, "<char>", "<char>");
      }
      break;
    }

    return 00;
}
