/*
 * Lao indirect list component.
 * 
 * LICENSE:
 *
 * Copyright (C) 2000 Leandro Bernsmller
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#include <stdio.h>
#include "../../include/sbase.h"
#include "../../include/lao/colist.h"
#include "../../include/lao/colist_e.h"



class_colist_indirect::class_colist_indirect ()
{
  init ();
}



class_colist_indirect::~class_colist_indirect ()
{
}



void  class_colist_indirect::
init ()
{
  class_colist::init ();
}



/*
 * Remove all elements from the list.
 */
void  class_colist_indirect::
terminate (class_enviroment * env, class_sbase_history* hist)
{
  class_colist::terminate (env, hist);
}



/*
 * Return the textual representation of the list.
 * env - enviroment
 * hist - history
 * return:
 * class_string pointer to string which represents the list
 */
DATAPT class_colist_indirect::
get_text_representation ( class_enviroment * env, class_sbase_history* hist )
{
  return class_colist::get_text_representation ('(', ')', env, hist);
}



/*
 * Activate the list acording the current activation mode.
 * self - class_component instance for this object
 * msg - lao system message ID
 * env - reference to lao name table (enviroment)
 * hist - reference to history
 * param - parameter to component
 * Return the activation result.
 */
DATAPT class_colist_indirect::
activate (class_component * self, DATAPT msg, class_enviroment * env,
          class_sbase_history* hist, DATAPT param )
{
    class_component* head_comp;
    BOOL success = get_first (put_in head_comp);

    if (success) {

        // activate the first component
        class_component* res_comp = (class_component*) head_comp->call (INVOKE_AS_REFERENCE, env, hist, (DATAPT) self);

        if (res_comp) {

                // get the text representation of the returned component
                class_string *res_str = (class_string*) res_comp->call (BIN_GET_TEXT_REPRESENTATION, env, hist, (DATAPT) self);

                // get the component associated with this name in enviroment
                class_component* comp_assoc = 00;
                DWORD num = env->get_value (res_str->get (), PARAMS_NAME, (class_component*) param, &comp_assoc);

                // verify if the associated component was found
                if ( num ) {
                  // activate it, and return the result component
                  return comp_assoc->call (INVOKE, env, hist, (DATAPT) self);
                }
                else {
                  // dbug - inform error in history and stop
                  printf ("Warning: component with name %s was not found, returning list component!\n", res_str->get());
                  return (DATAPT) self;
                }
        }
    }

    return (DATAPT) self;
}
