/*
 * Standard Lao component: makes program exit.
 * Parameters:
 * - integer component as the exit code
 * 
 * LICENSE:
 *
 * Copyright (C) 2000 Leandro Bernsmller
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



#include <stdio.h>
#include "../../include/lao.h"
#include "../../include/stdcomp/exit.h"



/*
 * Function to handle component messages.
 * self - class_component instance for this object
 * msg - lao system message ID
 * env - reference to lao name table
 * hist - reference to history
 * pdata - points component data
 * param - component parameters
 *
 * return:
 * message process result
 */
DATAPT
call_stdcomp_exit (class_component * self, DATAPT msg,
                   class_enviroment * env, class_sbase_history* hist,
                   DATAPT * pdata, DATAPT param)
{
        switch (msg) {

        case BIN_CREATE_DATA:
                break;

        case BIN_CLEAN_DATA:
                break;

        case INVOKE:
        case INVOKE_AS_REFERENCE:
          return self->call (ACTIVATE, env, hist, param);
          break;

        case ACTIVATE:
        case ACTIVATE_AS_REFERENCE:
                {
                  // REVIEW: deve obter o codigo de retorno dos parametros

		  DWORD numparams = paramc (param,env,hist);
		  DWORD excod = 0;
		  
		  if (numparams >= 2) {
		    class_component* comp_par = paramv (numparams - 1, param, env, hist);
		    excod = comp_par->call (set_msgsrv (BIN_ACTIVATE, GET_INTEGER), env, hist, 0);
		  }

                  hist->add_flag (LH_EXIT_CODE, excod);
                  hist->set_exception (LE_EXIT);
	        }
                break;

        case ACTIVATE_AS_PARAMETER:
                return (DATAPT) self;
                break;

        case BIN_ACTIVATE:
                break;

        case BIN_GET_TEXT_REPRESENTATION:
                {
                        class_string *str = (class_string *) malloc (sizeof (class_string));
                        str->init ();
                        str->append ("_exit_");
                        return (DATAPT) str;
                }
                break;

        case COPY:
                {
                   return (DATAPT) build_component ((DATAPT) 0, env, hist, ::call_stdcomp_exit, "<exit>", "<exit>");
                }
                break;
        }

        return (DATAPT) self;
}
